#!/usr/bin/env python3
# -*- coding:utf-8 -*
#

import os
import time
from shutil import copyfile

from sqlalchemy import Column, ForeignKey, Integer, String, Boolean, DateTime, Float, create_engine
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import relationship, backref, joinedload_all, Session
from sqlalchemy_utils import IPAddressType

engine = create_engine("sqlite:///base.db", echo=0)
dbsession = Session(bind=engine)


Base = declarative_base()

class Fingerprint(Base):
    __tablename__ = "fingerprint"
    id = Column(Integer, primary_key=True, nullable=False)
    location_id = Column(Integer, ForeignKey("location.id"), nullable=False)
    rssi_samples = relationship("RSSISample", cascade="all, delete-orphan", backref="fingerprint", uselist=False)

class Location(Base):
    __tablename__ = "location"
    id = Column(Integer, primary_key=True, nullable=False)
    x = Column(Float, nullable=False)
    y = Column(Float, nullable=False)
    fingerprint = relationship("Fingerprint", cascade="all, delete-orphan", backref="location")

    def __str__(self):
        return "id: %d (%f, %f)" % (self.id, self.x, self.y)

class RSSISample(Base):
    __tablename__ = "rssisample"
    id = Column(Integer, primary_key=True, nullable=False)
    fingerprint_id = Column(Integer, ForeignKey("fingerprint.id"), nullable=False)
    rssi_elements = relationship("RSSIElement", cascade="all, delete-orphan", backref="rssisample", lazy="dynamic")

    # TODO: define an iterator to be able to do: for rssi_elem in rssi_sample
    # we could use the RSSISample class as a list that way
    def items(self):
        return [(e.ip,e.value) for e in self.rssi_elements]

    def keys(self):
        return [e.ip for e in self.rssi_elements]

    def values(self):
        return [e.value for e in self.rssi_elements]

    def __getitem__(self, i):
        return self.rssi_elements.filter(RSSIElement.ip==i).first().value

class RSSIElement(Base):
    __tablename__ = "rssielement"
    id = Column(Integer, primary_key=True, nullable=False)
    value = Column(Integer, nullable=False)
    ip = Column(IPAddressType,nullable=False)
    rssi_sample_id = Column(Integer, ForeignKey("rssisample.id"), nullable=False)

class AccessPoint(Base):
    __tablename__ = "accesspoint"
    id = Column(Integer, primary_key=True, nullable=False)
    name = Column(String(32),nullable=False)
    ip_address = Column(IPAddressType,nullable=False)

    def __str__(self):
        return self.name + " - " + str(self.ip_address)

def setup():
    """
    Setup the database
    Makes a backup each time it is called, to ensure not loosing some precious data
    """
    if os.path.isfile("base.db"):
        copyfile("base.db", "base.db.backup."+str(time.time()))
    engine = create_engine("sqlite:///base.db", echo=1)
    Base.metadata.create_all(engine)
    dbsession = Session(engine)

def main():
    setup()
    #exp of adding to DB
    #loc = Location(x=10, y=42)
    #dbsession.add(loc)
    #dbsession.commit()

if __name__ == "__main__":
    setup()




